unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  uDragFilesSrc, StdCtrls, ExtCtrls, ComCtrls, uDragFilesTrg, ShellApi;

type
  TForm1 = class(TForm)
    Button1: TButton;
    ListView1: TListView;
    Edit1: TEdit;
    Button2: TButton;
    Panel1: TPanel;
    DragFilesSrc1: TDragFilesSrc;
    DragFilesTrg1: TDragFilesTrg;
    Label1: TLabel;
    Button3: TButton;
    procedure DragFilesSrc1Dropping(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure ListView1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ListView1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Button2Click(Sender: TObject);
    procedure DragFilesTrg1Drop(Sender: TObject);
    procedure Label1Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  private
    xDown, yDown: integer;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

//NOTE:
//  DragFilesSrc1 deals with dragging FROM listview.
//  DragFilesTrg1 deals with dragging TO listview.

// add the programme name to ListView.
procedure TForm1.FormCreate(Sender: TObject);
var NewItem : TListItem;
begin
  NewItem := ListView1.Items.Add;
  NewItem.Caption := paramstr(0);
end;

// set private x,y variables to detect start of drag (FROM) operation
procedure TForm1.ListView1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  xDown := x;
  yDown := y;
end;

procedure TForm1.ListView1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  DragResult: TDragResult;
  i: integer;
begin
// assesses whether is a drag (FROM) operation (ie: has the mouse moved > 20 pixels).
  if (not (ssLeft in Shift)) or (xDown = -1) or
     ((abs(xDown-x)<20) and (abs(yDown-y)<20)) or (ListView1.selected = nil) then exit;
  xDown := -1; {ie: don't do this again until another MouseDown event is called!}

  // Yes, it is a drag (FROM) operation.

  with DragFilesSrc1 do begin
    // Add files selected in ListView to DragFilesSrc1 list
    // Note: this does not Copy or Move the files anywhere yet!
    ClearFiles;
    for i := 1 to ListView1.items.count do
      if ListView1.items[i-1].selected then
        AddFile(ListView1.items[i-1].caption);

    if DropEffect = deCopy then
      Panel1.caption := 'Copying '+inttostr(FileCount)+' Files'
    else Panel1.caption := 'Moving '+inttostr(FileCount)+' Files';

    DragFilesTrg1.enabled := false;  //ListView1 should not accept files
                                     //while dragging from itself.

    // This is the START of the drag (FROM) operation.
    // Note:
    //   1. Copying or Moving will only occur IF dropped over a valid target.
    //   2. Copying or Moving will only happen AFTER any assigned OnDropping event.
    DragResult := Execute; {!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!}

    DragFilesTrg1.enabled  := true;  //ListView1 can accept files again.

    if DragResult = drInvalid then showmessage('Something went wrong!')
    else if DragResult = drCancelled then showmessage('Drag cancelled.')
    else showmessage('It''s Done!');
    Panel1.caption := '';
  end;
end;

procedure TForm1.DragFilesSrc1Dropping(Sender: TObject);
var
  OpStr: string;
begin
  // If files are to be extracted from an archive or downloaded
  // from the net do it here - into the Temp Directory - and
  // then move them from there to the Target Directory.

  ReleaseCapture;
  if DragFilesSrc1.DropEffect = deCopy then OpStr := 'COPY' else OpStr := 'MOVE';
  //Note: clearing the Files from DragFilesSrc1 cancels the operation
  if application.messagebox(PChar('You''re just about to '+OpStr+' the files!'#10+
    'Do you wish to continue?'), 'Drag & Drop Demo',
        mb_iconquestion+mb_yesno) <> idyes then DragFilesSrc1.clearfiles;
end;

// Add your own files to ListView by entering their names
// in Edit1 and then clicking this Button.
// (Otherwise drag them into the ListView from Explorer.)
procedure TForm1.Button2Click(Sender: TObject);
var
  NewItem : TListItem;
begin
  if (edit1.text = '') or not fileexists(edit1.text) then messagebeep(0)
  else begin
    NewItem := ListView1.Items.Add;
    NewItem.Caption := edit1.text;
    edit1.text := '';
  end;
  edit1.setfocus;
end;

// What to do when files are dropped on ListView.
procedure TForm1.DragFilesTrg1Drop(Sender: TObject);
var
  i: integer;
  NewItem : TListItem;
begin
  for i := 1 to DragFilesTrg1.FileCount do begin
    NewItem := ListView1.Items.Add;
    NewItem.Caption := uppercase(DragFilesTrg1.FileList[i-1]);
  end;
  with DragFilesTrg1.DropPoint do
    Panel1.caption := 'Dropped at point '+inttostr(x)+':'+inttostr(y)+
      ' on '+ DragFilesTrg1.Target.name;
end;

// Send me a message -  'Thanks!' , 'Something doesn't work!' etc.
procedure TForm1.Label1Click(Sender: TObject);
begin
  ShellExecute(0, Nil, 'mailto:ajohnson@rpi.net.au', Nil, Nil, SW_NORMAL);
end;

//Toggle between Copy & Move...
procedure TForm1.Button3Click(Sender: TObject);
begin
  if Button3.caption = 'C&opy' then begin
    Button3.caption := 'M&ove';
    DragFilesSrc1.DropEffect := deMove;
    end
  else begin
    Button3.caption := 'C&opy';
    DragFilesSrc1.DropEffect := deCopy;
  end;
end;

// OK, I've seen enough!
procedure TForm1.Button1Click(Sender: TObject);
begin
  close;
end;

end.
